<?php

namespace Drupal\ddos_security\EventSubscriber;

use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Symfony\Component\HttpKernel\KernelEvents;
use Symfony\Component\HttpKernel\Event\GetResponseEvent;
use Drupal\Core\Database\Driver\mysql\Connection;
use Drupal\Core\Session\AccountProxyInterface;
use Drupal\Core\Session\SessionManagerInterface;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Drupal\Core\Config\ConfigFactoryInterface;
use Symfony\Component\HttpFoundation\RequestStack;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\Core\Path\CurrentPathStack;
use Drupal\Core\Datetime\DateFormatterInterface;

/**
 * DDoS Attack Protection.
 */
class AttackProtection implements EventSubscriberInterface {

  /**
   * The client IP address.
   *
   * @var string
   */
  protected $clientIp;

  /**
   * The Config storage.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $config;

  /**
   * The Logger factory.
   *
   * @var \Drupal\Core\Logger\LoggerChannelFactoryInterface
   */
  protected $loggerFactory;

  /**
   * Drupal\Core\Database\Driver\mysql\Connection definition.
   *
   * @var \Drupal\Core\Database\Driver\mysql\Connection
   */
  protected $database;

  /**
   * The account.
   *
   * @var \Drupal\Core\Session\AccountProxyInterface
   */
  protected $account;

  /**
   * Current path.
   *
   * @var \Drupal\Core\Path\CurrentPathStack
   */
  protected $currentPath;

  /**
   * The date formatter service.
   *
   * @var \Drupal\Core\Datetime\DateFormatterInterface
   */
  protected $dateFormatter;

  /**
   * {@inheritdoc}
   */
  public function __construct(RequestStack $request_stack,
                              ConfigFactoryInterface $config,
                              LoggerChannelFactoryInterface $logger_factory,
                              Connection $database,
                              AccountProxyInterface $account,
                              SessionManagerInterface $session_manager,
                              CurrentPathStack $current_path,
                              DateFormatterInterface $date_formatter) {
    $this->clientIp = $request_stack->getCurrentRequest()->getClientIp();
    $this->config = $config;
    $this->loggerFactory = $logger_factory;
    $this->database = $database;
    $this->currentUser = $account;
    $this->sessionManager = $session_manager;
    $this->currentPath = $current_path;
    $this->dateFormatter = $date_formatter;
  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents() {
    // $events[KernelEvents::REQUEST][] = ['ddosSecurity', 31];
    $events[KernelEvents::REQUEST][] = ['ddosSecurity', 300];
    return $events;
  }

  /**
   * Call this method whenever the KernelEvents::TERMINATE event is dispatched.
   */
  public function ddosSecurity(GetResponseEvent $event) {
    $ddosConfig = $this->config->get('ddos_security.settings');
    $current_path = $this->currentPath->getPath();

    if ($this->currentUser->isAnonymous() && $ddosConfig->get('enable_ddos') == 1) {
      $ip_status = !empty(ddos_security_user_access_status()) ? ddos_security_user_access_status() : '';

      if ($ip_status != 'blocked' || strpos($current_path, '/user/') !== FALSE || $current_path == $ddosConfig->get('redirect_url')) {
        $t_time = date('ymdHis', time());
        $frm_time = date('ymdHis', strtotime("-1 minutes"));
        $created_date = date(time());

        $ip_count = $this->ipHitCount($frm_time, $t_time);

        if ($ip_count < $ddosConfig->get('total_hits')) {
          $query = $this->database->select('ddos_security');
          $query->condition('ip_address', $this->clientIp, '=');
          $query->addExpression('MAX(sno)');
          $max_sno = $query->execute()->fetchField();

          if ($max_sno < $t_time) {
            try {
              $allowed_ips = explode("\r\n", trim($ddosConfig->get('whitelisted_ip_addresses')));
              $allowed_pages = explode("\r\n", trim($ddosConfig->get('whitelisted_pages')));

              $log_data = ($current_path == $ddosConfig->get('redirect_url') || in_array($this->clientIp, $allowed_ips) || in_array($current_path, $allowed_pages)) ? 'allowed' : 'denied';

              if ($log_data == 'denied') {
                $ddos_insert = $this->database->insert('ddos_security')
                  ->fields(['sno', 'ip_address', 'status', 'created_date'])
                  ->values([
                    'sno' => $t_time,
                    'ip_address' => mb_substr($this->clientIp, 0, 128),
                    'status' => 'allowed',
                    'created_date' => $created_date,
                  ]);
                $ddos_insert->execute();
              }
            }
            catch (\Exception $e) {
              $this->loggerFactory->get('ddos_security')
                ->notice('<pre><code>Exception MESSAGE @error </code></pre>',
                [
                  '@error' => print_r($e, TRUE),
                ]);
            }
          }
        }

        else {
          try {
            $ddos_update = $this->database->update('ddos_security')
              ->fields(["status" => "blocked"])
              ->condition('ip_address', $this->clientIp, '=');
            $ddos_update->execute();
          }
          catch (\Exception $e) {
            $this->loggerFactory->get('ddos_security')
              ->notice('<pre><code>Exception MESSAGE @error </code></pre>',
              [
                '@error' => print_r($e, TRUE),
              ]);
          }

          $response = new RedirectResponse($ddosConfig->get('redirect_url'), 302);
          $response->send();
          return;
        }
      }

      else {
        $response = new RedirectResponse($ddosConfig->get('redirect_url'), 302);
        $response->send();
        return;
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function ipHitCount($frm_time = NULL, $t_time = NULL) {
    $ip_count = '';

    $query = $this->database->select("ddos_security", "ds");
    $query->condition('ds.ip_address', $this->clientIp, '=');
    $query->condition('ds.sno', $frm_time, '>=');
    $query->condition('ds.sno', $t_time, '<=');
    $query->condition('ds.status', 'allowed', '=');
    $query->addExpression('COUNT(*)');
    $ip_count = $query->execute()->fetchField();

    return $ip_count;
  }

}
